from xml.etree import cElementTree as eTree


def order_was_already_saved(
    order,  # type: eTree.XML
):
    # type: (...) -> bool

    state_history = order.findall(".//StateHistory/State")
    for state in state_history:
        if state.get("state") == "STORED":
            return True

    return False
