from messagebus import TokenCreator, TokenPriority


MSGGRP_SLCTRL_ORDERMGR = "A"

TK_SLCTRL_OMGR_DOSALE = TokenCreator.create_token(
    priority=TokenPriority.low,
    message_group=MSGGRP_SLCTRL_ORDERMGR,
    number="1",
)

TK_SLCTRL_OMGR_DOOPTION = TokenCreator.create_token(
    priority=TokenPriority.low,
    message_group=MSGGRP_SLCTRL_ORDERMGR,
    number="A",
)

TK_SLCTRL_OMGR_ORDERPICT = TokenCreator.create_token(
    priority=TokenPriority.low,
    message_group=MSGGRP_SLCTRL_ORDERMGR,
    number="2"
)

TK_SLCTRL_OMGR_CREATEORDER = TokenCreator.create_token(
    priority=TokenPriority.low,
    message_group=MSGGRP_SLCTRL_ORDERMGR,
    number="18"
)

TK_SLCTRL_OMGR_DOTENDER = TokenCreator.create_token(
    priority=TokenPriority.low,
    message_group=MSGGRP_SLCTRL_ORDERMGR,
    number="10",
)

TK_SLCTRL_OMGR_LISTORDERS = TokenCreator.create_token(
    priority=TokenPriority.low,
    message_group=MSGGRP_SLCTRL_ORDERMGR,
    number="D"
)

TK_SLCTRL_OMGR_CHANGEQTY = TokenCreator.create_token(
    priority=TokenPriority.low,
    message_group=MSGGRP_SLCTRL_ORDERMGR,
    number="E",
)

TK_SLCTRL_OMGR_ORDERREOPEN = TokenCreator.create_token(
    priority=TokenPriority.low,
    message_group=MSGGRP_SLCTRL_ORDERMGR,
    number="F",
)

TK_SLCTRL_OMGR_ORDERTOTAL = TokenCreator.create_token(
    priority=TokenPriority.low,
    message_group=MSGGRP_SLCTRL_ORDERMGR,
    number="6",
)

TK_SLCTRL_OMGR_PULLORDERS = TokenCreator.create_token(
    priority=TokenPriority.low,
    message_group=MSGGRP_SLCTRL_ORDERMGR,
    number="34",
)

TK_SLCTRL_OMGR_RECALLORDER = TokenCreator.create_token(
    priority=TokenPriority.low,
    message_group=MSGGRP_SLCTRL_ORDERMGR,
    number="C",
)

TK_SLCTRL_OMGR_STOREORDER = TokenCreator.create_token(
    priority=TokenPriority.low,
    message_group=MSGGRP_SLCTRL_ORDERMGR,
    number="B",
)

TK_SLCTRL_OMGR_VOIDLINE = TokenCreator.create_token(
    priority=TokenPriority.low,
    message_group=MSGGRP_SLCTRL_ORDERMGR,
    number="3",
)
TK_SLCTRL_OMGR_VOIDORDER = TokenCreator.create_token(
    priority=TokenPriority.low,
    message_group=MSGGRP_SLCTRL_ORDERMGR,
    number="4",
)

TK_SLCTRL_OMGR_CLEAROPTION = TokenCreator.create_token(
    priority=TokenPriority.low,
    message_group=MSGGRP_SLCTRL_ORDERMGR,
    number="7",
)

TK_SLCTRL_OMGR_DOOPTION = TokenCreator.create_token(
    priority=TokenPriority.low,
    message_group=MSGGRP_SLCTRL_ORDERMGR,
    number="A",
)

TK_SLCTRL_OMGR_CHANGEMODIFIER = TokenCreator.create_token(
    priority=TokenPriority.low,
    message_group=MSGGRP_SLCTRL_ORDERMGR,
    number="13",
)


def get_tokens():
    return (
        TK_SLCTRL_OMGR_CHANGEMODIFIER,
        TK_SLCTRL_OMGR_CHANGEQTY,
        TK_SLCTRL_OMGR_CLEAROPTION,
        TK_SLCTRL_OMGR_DOOPTION,
        TK_SLCTRL_OMGR_DOSALE,
        TK_SLCTRL_OMGR_DOTENDER,
        TK_SLCTRL_OMGR_ORDERPICT,
        TK_SLCTRL_OMGR_CREATEORDER,
        TK_SLCTRL_OMGR_LISTORDERS,
        TK_SLCTRL_OMGR_ORDERREOPEN,
        TK_SLCTRL_OMGR_ORDERTOTAL,
        TK_SLCTRL_OMGR_PULLORDERS,
        TK_SLCTRL_OMGR_RECALLORDER,
        TK_SLCTRL_OMGR_STOREORDER,
        TK_SLCTRL_OMGR_VOIDLINE,
        TK_SLCTRL_OMGR_VOIDORDER,
    )
