class OrderWaitingProcessingException(Exception):

    def __init__(
        self,       # type: OrderWaitingProcessingException
    ):
        # type: (...) -> None

        self.message = "Pedido na fila para processamento"
