import pyscripts

from mbcontextmessagehandler import MbContextMessageBus
from product_api import ProductService


def get_product_service():
    # type: (...) -> ProductService

    return ProductService(
        message_bus=MbContextMessageBus(mbcontext=pyscripts.mbcontext),
    )
