data = (
'hwen',    # 0x00
'hwenj',    # 0x01
'hwenh',    # 0x02
'hwed',    # 0x03
'hwel',    # 0x04
'hwelg',    # 0x05
'hwelm',    # 0x06
'hwelb',    # 0x07
'hwels',    # 0x08
'hwelt',    # 0x09
'hwelp',    # 0x0a
'hwelh',    # 0x0b
'hwem',    # 0x0c
'hweb',    # 0x0d
'hwebs',    # 0x0e
'hwes',    # 0x0f
'hwess',    # 0x10
'hweng',    # 0x11
'hwej',    # 0x12
'hwec',    # 0x13
'hwek',    # 0x14
'hwet',    # 0x15
'hwep',    # 0x16
'hweh',    # 0x17
'hwi',    # 0x18
'hwig',    # 0x19
'hwigg',    # 0x1a
'hwigs',    # 0x1b
'hwin',    # 0x1c
'hwinj',    # 0x1d
'hwinh',    # 0x1e
'hwid',    # 0x1f
'hwil',    # 0x20
'hwilg',    # 0x21
'hwilm',    # 0x22
'hwilb',    # 0x23
'hwils',    # 0x24
'hwilt',    # 0x25
'hwilp',    # 0x26
'hwilh',    # 0x27
'hwim',    # 0x28
'hwib',    # 0x29
'hwibs',    # 0x2a
'hwis',    # 0x2b
'hwiss',    # 0x2c
'hwing',    # 0x2d
'hwij',    # 0x2e
'hwic',    # 0x2f
'hwik',    # 0x30
'hwit',    # 0x31
'hwip',    # 0x32
'hwih',    # 0x33
'hyu',    # 0x34
'hyug',    # 0x35
'hyugg',    # 0x36
'hyugs',    # 0x37
'hyun',    # 0x38
'hyunj',    # 0x39
'hyunh',    # 0x3a
'hyud',    # 0x3b
'hyul',    # 0x3c
'hyulg',    # 0x3d
'hyulm',    # 0x3e
'hyulb',    # 0x3f
'hyuls',    # 0x40
'hyult',    # 0x41
'hyulp',    # 0x42
'hyulh',    # 0x43
'hyum',    # 0x44
'hyub',    # 0x45
'hyubs',    # 0x46
'hyus',    # 0x47
'hyuss',    # 0x48
'hyung',    # 0x49
'hyuj',    # 0x4a
'hyuc',    # 0x4b
'hyuk',    # 0x4c
'hyut',    # 0x4d
'hyup',    # 0x4e
'hyuh',    # 0x4f
'heu',    # 0x50
'heug',    # 0x51
'heugg',    # 0x52
'heugs',    # 0x53
'heun',    # 0x54
'heunj',    # 0x55
'heunh',    # 0x56
'heud',    # 0x57
'heul',    # 0x58
'heulg',    # 0x59
'heulm',    # 0x5a
'heulb',    # 0x5b
'heuls',    # 0x5c
'heult',    # 0x5d
'heulp',    # 0x5e
'heulh',    # 0x5f
'heum',    # 0x60
'heub',    # 0x61
'heubs',    # 0x62
'heus',    # 0x63
'heuss',    # 0x64
'heung',    # 0x65
'heuj',    # 0x66
'heuc',    # 0x67
'heuk',    # 0x68
'heut',    # 0x69
'heup',    # 0x6a
'heuh',    # 0x6b
'hyi',    # 0x6c
'hyig',    # 0x6d
'hyigg',    # 0x6e
'hyigs',    # 0x6f
'hyin',    # 0x70
'hyinj',    # 0x71
'hyinh',    # 0x72
'hyid',    # 0x73
'hyil',    # 0x74
'hyilg',    # 0x75
'hyilm',    # 0x76
'hyilb',    # 0x77
'hyils',    # 0x78
'hyilt',    # 0x79
'hyilp',    # 0x7a
'hyilh',    # 0x7b
'hyim',    # 0x7c
'hyib',    # 0x7d
'hyibs',    # 0x7e
'hyis',    # 0x7f
'hyiss',    # 0x80
'hying',    # 0x81
'hyij',    # 0x82
'hyic',    # 0x83
'hyik',    # 0x84
'hyit',    # 0x85
'hyip',    # 0x86
'hyih',    # 0x87
'hi',    # 0x88
'hig',    # 0x89
'higg',    # 0x8a
'higs',    # 0x8b
'hin',    # 0x8c
'hinj',    # 0x8d
'hinh',    # 0x8e
'hid',    # 0x8f
'hil',    # 0x90
'hilg',    # 0x91
'hilm',    # 0x92
'hilb',    # 0x93
'hils',    # 0x94
'hilt',    # 0x95
'hilp',    # 0x96
'hilh',    # 0x97
'him',    # 0x98
'hib',    # 0x99
'hibs',    # 0x9a
'his',    # 0x9b
'hiss',    # 0x9c
'hing',    # 0x9d
'hij',    # 0x9e
'hic',    # 0x9f
'hik',    # 0xa0
'hit',    # 0xa1
'hip',    # 0xa2
'hih',    # 0xa3
None,    # 0xa4
None,    # 0xa5
None,    # 0xa6
None,    # 0xa7
None,    # 0xa8
None,    # 0xa9
None,    # 0xaa
None,    # 0xab
None,    # 0xac
None,    # 0xad
None,    # 0xae
None,    # 0xaf
None,    # 0xb0
None,    # 0xb1
None,    # 0xb2
None,    # 0xb3
None,    # 0xb4
None,    # 0xb5
None,    # 0xb6
None,    # 0xb7
None,    # 0xb8
None,    # 0xb9
None,    # 0xba
None,    # 0xbb
None,    # 0xbc
None,    # 0xbd
None,    # 0xbe
None,    # 0xbf
None,    # 0xc0
None,    # 0xc1
None,    # 0xc2
None,    # 0xc3
None,    # 0xc4
None,    # 0xc5
None,    # 0xc6
None,    # 0xc7
None,    # 0xc8
None,    # 0xc9
None,    # 0xca
None,    # 0xcb
None,    # 0xcc
None,    # 0xcd
None,    # 0xce
None,    # 0xcf
None,    # 0xd0
None,    # 0xd1
None,    # 0xd2
None,    # 0xd3
None,    # 0xd4
None,    # 0xd5
None,    # 0xd6
None,    # 0xd7
None,    # 0xd8
None,    # 0xd9
None,    # 0xda
None,    # 0xdb
None,    # 0xdc
None,    # 0xdd
None,    # 0xde
None,    # 0xdf
None,    # 0xe0
None,    # 0xe1
None,    # 0xe2
None,    # 0xe3
None,    # 0xe4
None,    # 0xe5
None,    # 0xe6
None,    # 0xe7
None,    # 0xe8
None,    # 0xe9
None,    # 0xea
None,    # 0xeb
None,    # 0xec
None,    # 0xed
None,    # 0xee
None,    # 0xef
None,    # 0xf0
None,    # 0xf1
None,    # 0xf2
None,    # 0xf3
None,    # 0xf4
None,    # 0xf5
None,    # 0xf6
None,    # 0xf7
None,    # 0xf8
None,    # 0xf9
None,    # 0xfa
None,    # 0xfb
None,    # 0xfc
None,    # 0xfd
None,    # 0xfe
)
