from dailygoals_services.model.repository import DailyGoalsRepository


class DeleteSalesChallengeInteractor:
    def __init__(self, daily_goals_repository):
        # type: (DailyGoalsRepository) -> None

        self.daily_goals_repository = daily_goals_repository

    def delete_sales_challenge(self, sale_challenge_id):
        # type: (int) -> int

        return self.daily_goals_repository.delete_sales_challenge(sale_challenge_id)
